# Smart Paste Developer Documentation

## Architecture Overview

The Smart Paste module consists of several key components:

### Core Components

1. **SmartPasteService** (`src/Service/SmartPasteService.php`)
   - Main service handling AI integration and form button management
   - Manages configuration and AI provider communication
   - Generates prompts for AI models

2. **SmartPasteController** (`src/Controller/SmartPasteController.php`)
   - AJAX endpoint for data extraction
   - Handles JSON requests from frontend JavaScript
   - Returns structured data or error responses

3. **SmartPasteAdminForm** (`src/Form/SmartPasteAdminForm.php`)
   - Configuration form for module settings
   - AI provider and model selection
   - Form targeting configuration

4. **JavaScript Integration** (`js/smart-paste.js`)
   - Client-side form analysis
   - Clipboard API integration
   - Dynamic form field population

### Data Flow

```
User Clipboard → JavaScript Analysis → AJAX Request → AI Provider →
Structured Data → Form Population
```

1. User copies unstructured data to clipboard
2. JavaScript analyzes form structure when Smart Paste button is clicked
3. Form structure and clipboard data sent to server endpoint
4. Server generates AI prompt and calls configured AI provider
5. AI returns structured JSON data
6. JavaScript populates form fields with extracted data

## Extending the Module

### Adding Custom Field Type Support

To support additional field types, modify the `fillFormFields` function in
`js/smart-paste.js`:

```javascript
case 'custom-field-type':
  // Handle custom field population logic
  customFieldHandler(field, value);
  break;
```

### Custom AI Prompt Generation

Extend the `SmartPasteService::generatePrompt()` method:

```php
protected function generatePrompt(
  array $form_fields,
  string $unstructured_data
): string {
  $prompt = parent::generatePrompt($form_fields, $unstructured_data);

  // Add custom prompt modifications
  $prompt .= "\nAdditional instructions for specific field types...";

  return $prompt;
}
```

### Form Field Analysis Enhancement

Modify the `analyzeFormFields` function in JavaScript to better detect field
types:

```javascript
// Enhanced field detection
if (input.classList.contains('custom-field-class')) {
  fieldInfo.type = 'custom-type';
  fieldInfo.customAttribute = input.getAttribute('data-custom');
}
```

## Configuration

### AI Provider Integration

The module integrates with any AI provider supported by the AI module:

- OpenAI GPT models
- Google Gemini
- Anthropic Claude
- Local models via Ollama
- Custom providers

### Form Targeting

Configure which forms should have Smart Paste buttons:

```yaml
# config/install/smart_paste.settings.yml
enabled: true
form_mode: 'all'  # or 'specific'
allowed_forms:
  - 'node_article_form'
  - 'user_register_form'
excluded_forms:
  - 'user_login_form'
  - 'search_form'
```

## Security Considerations

### CSRF Protection

All AJAX requests include CSRF tokens:

```javascript
headers: {
  'X-CSRF-Token': drupalSettings.smart_paste.csrf_token,
}
```

### Permission Checks

Multiple permission layers:

1. Module permission: `use smart paste`
2. Form-level checks in `shouldAddButton()`
3. CSRF validation on AJAX endpoints

### Data Handling

- No permanent storage of clipboard data
- AI responses are not cached
- Error messages don't expose sensitive information

## Testing

### Functional Tests

Run tests with PHPUnit:

```bash
vendor/bin/phpunit web/modules/contrib/smart_paste/tests/
```

### Manual Testing

1. Enable module and configure AI provider
2. Test on various form types
3. Verify clipboard integration works
4. Check error handling with invalid data

## Debugging

### Common Issues

1. **Smart Paste button not appearing**
   - Check module is enabled
   - Verify user has `use smart paste` permission
   - Ensure form is not in excluded list

2. **AI extraction fails**
   - Check AI provider configuration
   - Verify API keys are set
   - Check Drupal logs for errors

3. **Form fields not filling**
   - Check browser console for JavaScript errors
   - Verify clipboard contains text
   - Ensure form fields have proper names

### Debug Mode

Enable debug logging:

```php
// In settings.php
$config['system.logging']['error_level'] = 'verbose';
```

Check logs at `/admin/reports/dblog`.

## Performance Considerations

### Caching

- AI responses are not cached (for security)
- Form analysis is done client-side
- Configuration is cached via Drupal's config system

### Optimization

- JavaScript only loads on forms with Smart Paste buttons
- AJAX requests include only necessary data
- Error handling prevents unnecessary retries

## Contributing

### Code Standards

Follow Drupal coding standards:

```bash
vendor/bin/phpcs --standard=Drupal web/modules/contrib/smart_paste/
```

### Adding Features

1. Create feature branch
2. Add functionality with tests
3. Update documentation
4. Submit patch to Drupal.org

## API Reference

### Services

- `smart_paste.service`: Main Smart Paste functionality
- `smart_paste.controller`: AJAX endpoints

### Permissions

- `administer smart paste`: Configure module
- `use smart paste`: Use functionality

### Configuration

- `smart_paste.settings`: Main configuration object

### JavaScript API

```javascript
// Access Smart Paste functionality
if (typeof Drupal.behaviors.smartPaste !== 'undefined') {
  // Smart Paste is available
}
```
