/**
 * @file
 * Smart Paste JavaScript functionality.
 */

(function smartPasteModule(Drupal, drupalSettings) {
  /**
   * Shows status message.
   *
   * @param {string} message - The message to display.
   * @param {string} type - The type of message (status, error, success).
   */
  function showStatus(message, type = 'status') {
    const statusContainer = document.querySelector('.smart-paste-status');
    if (!statusContainer) {
      return;
    }

    statusContainer.innerHTML = `<div class="messages messages--${type}">${message}</div>`;
    statusContainer.style.display = 'block';
  }

  /**
   * Hides status message.
   */
  function hideStatus() {
    const statusContainer = document.querySelector('.smart-paste-status');
    if (statusContainer) {
      statusContainer.style.display = 'none';
      statusContainer.innerHTML = '';
    }
  }

  /**
   * Analyzes form fields to extract their structure.
   *
   * @param {HTMLElement} formElement - The form element to analyze.
   * @return {Object} Object containing field information.
   */
  function analyzeFormFields(formElement) {
    const fields = {};

    // Get all form input elements.
    const inputs = formElement.querySelectorAll('input, select, textarea');

    inputs.forEach((input) => {
      const { name } = input;
      let type = input.type || input.tagName.toLowerCase();

      // Skip certain input types.
      if (
        !name ||
        type === 'hidden' ||
        type === 'submit' ||
        type === 'button' ||
        name.includes('form_') ||
        name.includes('op') ||
        input.classList.contains('smart-paste-button')
      ) {
        return;
      }

      // Handle text_format fields specially
      let fieldName = name;

      if (name.endsWith('[value]') && type === 'textarea') {
        // This is the main textarea of a text_format field
        fieldName = name.replace('[value]', '');
        type = 'text_format';
      } else if (name.endsWith('[format]')) {
        // Skip the format selector part of text_format fields
        // We already handle these through the [value] part
        return;
      }

      // Get field label.
      let label = '';
      const labelElement = formElement.querySelector(
        `label[for="${input.id}"]`,
      );
      if (labelElement) {
        label = labelElement.textContent.trim();
      } else {
        // Try to find label by looking at nearby elements.
        const wrapper = input.closest(
          '.form-item, .field--widget, .form-element',
        );
        if (wrapper) {
          const wrapperLabel = wrapper.querySelector('label');
          if (wrapperLabel) {
            label = wrapperLabel.textContent.trim();
          }
        }
      }

      // Handle different field types.
      const fieldInfo = {
        type,
        label: label || fieldName,
      };

      // Get options for select and radio fields.
      if (
        type === 'select' ||
        type === 'select-one' ||
        type === 'select-multiple'
      ) {
        const options = [];
        input.querySelectorAll('option').forEach((option) => {
          if (option.value && option.value !== '') {
            options.push(option.textContent.trim());
          }
        });
        fieldInfo.options = options;
      } else if (type === 'radio') {
        // Group radio buttons by name.
        if (!fields[fieldName]) {
          const radioButtons = formElement.querySelectorAll(
            `input[name="${name}"]`,
          );
          const options = [];
          radioButtons.forEach((radio) => {
            const radioLabel = formElement.querySelector(
              `label[for="${radio.id}"]`,
            );
            if (radioLabel) {
              options.push(radioLabel.textContent.trim());
            } else {
              options.push(radio.value);
            }
          });
          fieldInfo.options = options;
        } else {
          return; // Skip duplicate radio buttons.
        }
      }

      fields[fieldName] = fieldInfo;
    });

    return fields;
  }

  /**
   * Sends data to server for AI processing.
   *
   * @param {string} unstructuredData - The unstructured data to process.
   * @param {Object} formFields - The form fields structure.
   * @return {Promise} Promise that resolves with extracted data.
   */
  function extractDataWithAI(unstructuredData, formFields) {
    const url = drupalSettings.smart_paste.extract_url;

    return fetch(url, {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        'X-CSRF-Token': drupalSettings.smart_paste.csrf_token,
      },
      body: JSON.stringify({
        unstructured_data: unstructuredData,
        form_fields: formFields,
      }),
    })
      .then((response) => {
        if (!response.ok) {
          throw new Error(`HTTP error! status: ${response.status}`);
        }
        return response.json();
      })
      .then((data) => {
        if (!data.success) {
          throw new Error(data.error || 'Unknown error occurred');
        }
        return data.data;
      });
  }

  /**
   * Fills form fields with extracted data.
   *
   * @param {HTMLElement} formElement - The form element to fill.
   * @param {Object} extractedData - The data to fill into the form.
   */
  function fillFormFields(formElement, extractedData) {
    Object.entries(extractedData).forEach(([fieldName, value]) => {
      // First try to find a text_format field (with [value] suffix)
      let field = formElement.querySelector(`[name="${fieldName}[value]"]`);
      const isTextFormatField = !!field;

      // If not found, try the regular field name
      if (!field) {
        field = formElement.querySelector(`[name="${fieldName}"]`);
      }

      if (!field) {
        return;
      }

      const fieldType = field.type || field.tagName.toLowerCase();

      // Handle text_format fields specially
      if (isTextFormatField || fieldType === 'text_format') {
        // First try to find and use CKEditor instance
        let ckEditorSet = false;

        // Method 1: Try to find CKEditor instance by field ID
        if (
          field.id &&
          typeof window.CKEDITOR !== 'undefined' &&
          window.CKEDITOR.instances &&
          window.CKEDITOR.instances[field.id]
        ) {
          window.CKEDITOR.instances[field.id].setData(value);
          ckEditorSet = true;
        }

        // Method 2: Try CKEditor 5 approach
        if (!ckEditorSet && field.id) {
          const wrapper = field.closest(
            '.form-item, .field--widget, .js-form-item',
          );
          if (wrapper) {
            const ckEditorDiv = wrapper.querySelector(
              '.ck-editor__editable[contenteditable="true"]',
            );
            if (
              ckEditorDiv &&
              typeof ckEditorDiv.ckeditorInstance !== 'undefined'
            ) {
              ckEditorDiv.ckeditorInstance.setData(value);
              ckEditorSet = true;
            }
          }
        }

        // Method 3: Fallback to setting textarea value directly
        if (!ckEditorSet) {
          field.value = value;
        }

        // Trigger change event for text_format fields
        field.dispatchEvent(new Event('change', { bubbles: true }));
        return;
      }

      switch (fieldType) {
        case 'text':
        case 'email':
        case 'tel':
        case 'url':
        case 'textarea': {
          field.value = value;
          break;
        }

        case 'select':
        case 'select-one': {
          // Find matching option.
          const option = Array.from(field.options).find(
            (opt) =>
              opt.textContent.trim().toLowerCase() === value.toLowerCase() ||
              opt.value.toLowerCase() === value.toLowerCase(),
          );
          if (option) {
            field.value = option.value;
          }
          break;
        }

        case 'radio': {
          // Find matching radio button.
          const radioButtons = formElement.querySelectorAll(
            `input[name="${fieldName}"]`,
          );
          radioButtons.forEach((radio) => {
            const radioLabel = formElement.querySelector(
              `label[for="${radio.id}"]`,
            );
            const labelText = radioLabel
              ? radioLabel.textContent.trim()
              : radio.value;
            if (
              labelText.toLowerCase() === value.toLowerCase() ||
              radio.value.toLowerCase() === value.toLowerCase()
            ) {
              radio.checked = true;
            }
          });
          break;
        }

        case 'checkbox': {
          if (typeof value === 'boolean') {
            field.checked = value;
          } else {
            // Try to parse as boolean.
            const boolValue = value.toString().toLowerCase();
            field.checked =
              boolValue === 'true' || boolValue === '1' || boolValue === 'yes';
          }
          break;
        }

        default: {
          // Try setting as value for unknown types.
          try {
            field.value = value;
          } catch (e) {
            // Silent fail for unsupported field types
          }
          break;
        }
      }

      // Trigger change event to notify other scripts.
      field.dispatchEvent(new Event('change', { bubbles: true }));
    });
  }

  /**
   * Handles Smart Paste button click.
   *
   * @param {Event} event - The click event.
   */
  function handleSmartPasteClick(event) {
    event.preventDefault();

    const button = event.target;
    let formElement = button.closest('form');

    // If we can't find the form (e.g., button is in Gin's "more actions" dropdown),
    // try alternative methods to find the form
    if (!formElement) {
      // Try to find form by data attribute
      const formId = button.getAttribute('data-form-id');
      if (formId && formId !== 'drupal-form') {
        formElement = document.getElementById(formId);
      }

      // If still no form, look for the main content form on the page
      if (!formElement) {
        // Common selectors for main content forms
        const formSelectors = [
          'form[data-drupal-selector*="node-"]', // Node forms
          'form[data-drupal-selector*="edit-"]', // Edit forms
          'form.node-form', // Node form class
          'form[id*="node-"]', // Forms with node in ID
          'form[id*="edit-"]', // Edit forms by ID
          'form[action*="node"]', // Forms with node in action
          'main form', // Main content form
          '.region-content form', // Form in content region
          'form:not([data-drupal-selector*="search"]):not([data-drupal-selector*="views-exposed"])', // Exclude search/filter forms
        ];

        // Use Array.some instead of for...of to avoid restricted syntax
        formSelectors.some((selector) => {
          formElement = document.querySelector(selector);
          return formElement !== null;
        });
      }
    }

    if (!formElement) {
      showStatus(
        'Error: Could not find form element. Please ensure the Smart Paste button is associated with a form.',
        'error',
      );
      return;
    }

    // Show loading state.
    button.disabled = true;
    button.textContent = 'Processing...';
    showStatus('Reading clipboard...', 'status');

    // Read clipboard data.
    navigator.clipboard
      .read()
      .then(async (clipboardItems) => {
        let clipboardData = '';
        let hasHtml = false;

        // Process clipboard items to find the best content
        // Use Promise.all to handle async operations properly
        const htmlItems = clipboardItems.filter((item) =>
          item.types.includes('text/html'),
        );

        if (htmlItems.length > 0 && !hasHtml) {
          try {
            const htmlBlob = await htmlItems[0].getType('text/html');
            const htmlText = await htmlBlob.text();
            clipboardData = htmlText;
            hasHtml = true;
          } catch (e) {
            // HTML reading failed, continue to plain text
          }
        }

        // If no HTML found, look for plain text
        if (!hasHtml) {
          const textItems = clipboardItems.filter((item) =>
            item.types.includes('text/plain'),
          );

          if (textItems.length > 0) {
            try {
              const textBlob = await textItems[0].getType('text/plain');
              const plainText = await textBlob.text();
              clipboardData = plainText;
            } catch (e) {
              // Plain text reading failed
            }
          }
        }

        if (!clipboardData || clipboardData.trim() === '') {
          throw new Error('Clipboard is empty or contains no text.');
        }

        showStatus('Analyzing form fields...', 'status');

        // Analyze form fields.
        const formFields = analyzeFormFields(formElement);

        if (Object.keys(formFields).length === 0) {
          throw new Error('No suitable form fields found.');
        }

        showStatus('Extracting data with AI...', 'status');

        // Send data to server for AI processing.
        return extractDataWithAI(clipboardData, formFields);
      })
      .catch(() => {
        // If clipboard.read() fails, fallback to readText()
        return navigator.clipboard.readText().then((textData) => {
          if (!textData || textData.trim() === '') {
            throw new Error('Clipboard is empty or contains no text.');
          }

          showStatus('Analyzing form fields...', 'status');

          // Analyze form fields.
          const formFields = analyzeFormFields(formElement);

          if (Object.keys(formFields).length === 0) {
            throw new Error('No suitable form fields found.');
          }

          showStatus('Extracting data with AI...', 'status');

          // Send data to server for AI processing.
          return extractDataWithAI(textData, formFields);
        });
      })
      .then((extractedData) => {
        showStatus('Filling form fields...', 'status');

        // Fill form fields with extracted data.
        fillFormFields(formElement, extractedData);

        showStatus('Smart Paste completed successfully!', 'success');

        // Reset button after a delay.
        setTimeout(() => {
          hideStatus();
        }, 3000);
      })
      .catch((error) => {
        showStatus(`Error: ${error.message}`, 'error');
      })
      .finally(() => {
        // Reset button state.
        button.disabled = false;
        button.textContent = 'Smart Paste';
      });
  }

  /**
   * Smart Paste behavior.
   */
  Drupal.behaviors.smartPaste = {
    attach(context) {
      const buttons = context.querySelectorAll(
        '.smart-paste-button:not(.smart-paste-processed)',
      );

      buttons.forEach((button) => {
        button.classList.add('smart-paste-processed');
        button.addEventListener('click', handleSmartPasteClick);
      });
    },
  };
})(Drupal, drupalSettings);
