<?php

namespace Drupal\smart_paste\Controller;

use Drupal\Core\Access\CsrfTokenGenerator;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\smart_paste\Service\SmartPasteService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Controller for Smart Paste AJAX endpoints.
 */
class SmartPasteController extends ControllerBase {

  /**
   * The Smart Paste service.
   *
   * @var \Drupal\smart_paste\Service\SmartPasteService
   */
  protected $smartPasteService;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The CSRF token generator.
   *
   * @var \Drupal\Core\Access\CsrfTokenGenerator
   */
  protected $csrfToken;

  /**
   * Constructs a SmartPasteController object.
   *
   * @param \Drupal\smart_paste\Service\SmartPasteService $smart_paste_service
   *   The Smart Paste service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\Access\CsrfTokenGenerator $csrf_token
   *   The CSRF token generator.
   */
  public function __construct(
    SmartPasteService $smart_paste_service,
    RequestStack $request_stack,
    LoggerChannelFactoryInterface $logger_factory,
    CsrfTokenGenerator $csrf_token,
  ) {
    $this->smartPasteService = $smart_paste_service;
    $this->requestStack = $request_stack;
    $this->logger = $logger_factory->get('smart_paste');
    $this->csrfToken = $csrf_token;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('smart_paste.service'),
      $container->get('request_stack'),
      $container->get('logger.factory'),
      $container->get('csrf_token')
    );
  }

  /**
   * Extracts data from unstructured text using AI.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with extracted data or error.
   */
  public function extractData(Request $request): JsonResponse {
    try {
      // Validate request method.
      if (!$request->isMethod('POST')) {
        return new JsonResponse(['error' => 'Method not allowed'], 405);
      }

      // Validate CSRF token.
      $csrf_token = $request->headers->get('X-CSRF-Token');
      if (!$csrf_token || !$this->csrfToken->validate($csrf_token, 'smart_paste_extract')) {
        return new JsonResponse(['error' => 'Invalid CSRF token'], 403);
      }

      // Get request data.
      $content = json_decode($request->getContent(), TRUE);

      if (json_last_error() !== JSON_ERROR_NONE) {
        return new JsonResponse(['error' => 'Invalid JSON'], 400);
      }

      $unstructured_data = $content['unstructured_data'] ?? '';
      $form_fields = $content['form_fields'] ?? [];

      // Validate input.
      if (empty($unstructured_data)) {
        return new JsonResponse(['error' => 'No data provided'], 400);
      }

      if (empty($form_fields)) {
        return new JsonResponse(['error' => 'No form fields provided'], 400);
      }

      // Check if Smart Paste is enabled.
      if (!$this->smartPasteService->isEnabled()) {
        return new JsonResponse(['error' => 'Smart Paste is not enabled'], 403);
      }

      // Extract data using AI.
      $extracted_data = $this->smartPasteService->extractData($unstructured_data, $form_fields);

      return new JsonResponse([
        'success' => TRUE,
        'data' => $extracted_data,
      ]);

    }
    catch (\Exception $e) {
      $this->logger->error('Smart Paste extraction failed: @error', ['@error' => $e->getMessage()]);

      return new JsonResponse([
        'success' => FALSE,
        'error' => 'Failed to extract data: ' . $e->getMessage(),
      ], 500);
    }
  }

}
