<?php

namespace Drupal\smart_paste\Form;

use Drupal\ai\AiProviderPluginManager;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Smart Paste settings.
 */
class SmartPasteAdminForm extends ConfigFormBase {

  /**
   * The AI provider plugin manager.
   *
   * @var \Drupal\ai\AiProviderPluginManager
   */
  protected $aiProviderManager;

  /**
   * Constructs a SmartPasteAdminForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typed_config_manager
   *   The typed config manager.
   * @param \Drupal\ai\AiProviderPluginManager $ai_provider_manager
   *   The AI provider plugin manager.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    TypedConfigManagerInterface $typed_config_manager,
    AiProviderPluginManager $ai_provider_manager,
  ) {
    parent::__construct($config_factory, $typed_config_manager);
    $this->aiProviderManager = $ai_provider_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('ai.provider')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['smart_paste.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'smart_paste_admin_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('smart_paste.settings');

    $form['enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Smart Paste'),
      '#description' => $this->t('When enabled, Smart Paste buttons will be added to forms based on the configuration below.'),
      '#default_value' => $config->get('enabled'),
    ];

    // AI Provider configuration.
    $form['ai_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('AI Configuration'),
      '#states' => [
        'visible' => [
          ':input[name="enabled"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Get available AI providers.
    $providers = [];
    $provider_definitions = $this->aiProviderManager->getDefinitions();
    foreach ($provider_definitions as $provider_id => $definition) {
      $provider = $this->aiProviderManager->createInstance($provider_id);
      if ($provider->isUsable('chat')) {
        $providers[$provider_id] = $definition['label'];
      }
    }

    $form['ai_settings']['ai_provider'] = [
      '#type' => 'select',
      '#title' => $this->t('AI Provider'),
      '#description' => $this->t('Select the AI provider to use for Smart Paste.'),
      '#options' => $providers,
      '#default_value' => $config->get('ai_provider'),
      '#required' => TRUE,
      '#ajax' => [
        'callback' => '::updateModels',
        'wrapper' => 'ai-models-wrapper',
      ],
    ];

    $form['ai_settings']['ai_model'] = [
      '#type' => 'select',
      '#title' => $this->t('AI Model'),
      '#description' => $this->t('Select the AI model to use for Smart Paste.'),
      '#options' => $this->getModelsForProvider($form_state->getValue('ai_provider') ?? $config->get('ai_provider')),
      '#default_value' => $config->get('ai_model'),
      '#required' => TRUE,
      '#prefix' => '<div id="ai-models-wrapper">',
      '#suffix' => '</div>',
    ];

    // Form targeting configuration.
    $form['form_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Form Configuration'),
      '#states' => [
        'visible' => [
          ':input[name="enabled"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['form_settings']['form_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Form Targeting Mode'),
      '#description' => $this->t('Choose how to determine which forms should have Smart Paste buttons.'),
      '#options' => [
        'all' => $this->t('All forms (except excluded)'),
        'specific' => $this->t('Only specific forms'),
      ],
      '#default_value' => $config->get('form_mode') ?? 'all',
    ];

    $form['form_settings']['allowed_forms'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Allowed Forms'),
      '#description' => $this->t('Enter form IDs (one per line) that should have Smart Paste buttons. Only used when "Only specific forms" is selected.'),
      '#default_value' => implode("\n", $config->get('allowed_forms') ?? []),
      '#states' => [
        'visible' => [
          ':input[name="form_mode"]' => ['value' => 'specific'],
        ],
      ],
    ];

    $form['form_settings']['excluded_forms'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Excluded Forms'),
      '#description' => $this->t('Enter form IDs (one per line) that should NOT have Smart Paste buttons. Only used when "All forms (except excluded)" is selected.'),
      '#default_value' => implode("\n", $config->get('excluded_forms') ?? []),
      '#states' => [
        'visible' => [
          ':input[name="form_mode"]' => ['value' => 'all'],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * AJAX callback to update models when provider changes.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The updated models element.
   */
  public function updateModels(array $form, FormStateInterface $form_state) {
    $provider_id = $form_state->getValue('ai_provider');
    $form['ai_settings']['ai_model']['#options'] = $this->getModelsForProvider($provider_id);
    return $form['ai_settings']['ai_model'];
  }

  /**
   * Gets available models for a provider.
   *
   * @param string|null $provider_id
   *   The provider ID.
   *
   * @return array
   *   Array of available models.
   */
  protected function getModelsForProvider(?string $provider_id): array {
    if (empty($provider_id)) {
      return [];
    }

    try {
      $provider = $this->aiProviderManager->createInstance($provider_id);
      return $provider->getConfiguredModels('chat');
    }
    catch (\Exception $e) {
      return [];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    if ($form_state->getValue('enabled')) {
      // Validate AI provider and model.
      $provider_id = $form_state->getValue('ai_provider');
      $model_id = $form_state->getValue('ai_model');

      if (empty($provider_id)) {
        $form_state->setErrorByName('ai_provider', $this->t('AI Provider is required when Smart Paste is enabled.'));
      }

      if (empty($model_id)) {
        $form_state->setErrorByName('ai_model', $this->t('AI Model is required when Smart Paste is enabled.'));
      }

      // Test AI provider availability.
      if (!empty($provider_id)) {
        try {
          $provider = $this->aiProviderManager->createInstance($provider_id);
          if (!$provider->isUsable('chat')) {
            $form_state->setErrorByName('ai_provider', $this->t('Selected AI provider is not available for chat operations.'));
          }
        }
        catch (\Exception $e) {
          $form_state->setErrorByName('ai_provider', $this->t('Error testing AI provider: @error', ['@error' => $e->getMessage()]));
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('smart_paste.settings');

    $config->set('enabled', $form_state->getValue('enabled'));
    $config->set('ai_provider', $form_state->getValue('ai_provider'));
    $config->set('ai_model', $form_state->getValue('ai_model'));
    $config->set('form_mode', $form_state->getValue('form_mode'));

    // Process allowed forms.
    $allowed_forms = array_filter(array_map('trim', explode("\n", $form_state->getValue('allowed_forms'))));
    $config->set('allowed_forms', $allowed_forms);

    // Process excluded forms.
    $excluded_forms = array_filter(array_map('trim', explode("\n", $form_state->getValue('excluded_forms'))));
    $config->set('excluded_forms', $excluded_forms);

    $config->save();

    parent::submitForm($form, $form_state);
  }

}
