<?php

namespace Drupal\smart_paste\Service;

use Drupal\ai\AiProviderPluginManager;
use Drupal\ai\OperationType\Chat\ChatInput;
use Drupal\ai\OperationType\Chat\ChatMessage;
use Drupal\Core\Access\CsrfTokenGenerator;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Routing\UrlGeneratorInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;

/**
 * Smart Paste service for handling AI-powered form filling.
 */
class SmartPasteService {

  use StringTranslationTrait;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The AI provider plugin manager.
   *
   * @var \Drupal\ai\AiProviderPluginManager
   */
  protected $aiProviderManager;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current user service.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The URL assembler service.
   *
   * @var \Drupal\Core\Routing\UrlGeneratorInterface
   */
  protected $urlGenerator;

  /**
   * The CSRF token generator.
   *
   * @var \Drupal\Core\Access\CsrfTokenGenerator
   */
  protected $csrfToken;

  /**
   * Constructs a SmartPasteService object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\ai\AiProviderPluginManager $ai_provider_manager
   *   The AI provider plugin manager.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user service.
   * @param \Drupal\Core\Routing\UrlGeneratorInterface $url_generator
   *   The URL generator service.
   * @param \Drupal\Core\Access\CsrfTokenGenerator $csrf_token
   *   The CSRF token generator.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translation service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    AiProviderPluginManager $ai_provider_manager,
    LoggerChannelFactoryInterface $logger_factory,
    EntityTypeManagerInterface $entity_type_manager,
    AccountProxyInterface $current_user,
    UrlGeneratorInterface $url_generator,
    CsrfTokenGenerator $csrf_token,
    TranslationInterface $string_translation,
  ) {
    $this->configFactory = $config_factory;
    $this->aiProviderManager = $ai_provider_manager;
    $this->logger = $logger_factory->get('smart_paste');
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $current_user;
    $this->urlGenerator = $url_generator;
    $this->csrfToken = $csrf_token;
    $this->setStringTranslation($string_translation);
  }

  /**
   * Checks if Smart Paste is enabled.
   *
   * @return bool
   *   TRUE if enabled, FALSE otherwise.
   */
  public function isEnabled(): bool {
    $config = $this->configFactory->get('smart_paste.settings');
    return $config->get('enabled') ?? FALSE;
  }

  /**
   * Checks if a form should have the Smart Paste button.
   *
   * @param string $form_id
   *   The form ID.
   *
   * @return bool
   *   TRUE if the button should be added, FALSE otherwise.
   */
  public function shouldAddButton(string $form_id): bool {
    $config = $this->configFactory->get('smart_paste.settings');

    // Built-in exclusions for security and usability.
    $builtin_excluded_forms = [
      // Authentication forms.
      'user_login_form',
      'user_pass',
      'user_pass_reset',
      // Search forms.
      'search_block_form',
      'search_form',
      'views_exposed_form',
      // Filter and admin forms.
      'views_bulk_operations_form',
      'views_form',
      'admin_toolbar_tools_flush_cache_link',
      // System forms.
      'system_modules',
      'system_theme_settings',
      'user_admin_permissions',
    ];

    // Check built-in exclusions first.
    if (in_array($form_id, $builtin_excluded_forms)) {
      return FALSE;
    }

    // Check for form ID patterns that should be excluded.
    $excluded_patterns = [
    // All search forms.
      '/^search_/',
    // Views exposed filter forms.
      '/^views_exposed_/',
    // Views bulk operation forms.
      '/^views_bulk_/',
    // Admin toolbar forms.
      '/^admin_toolbar_/',
    // Module management forms.
      '/^system_modules/',
    // Forms ending with filter_form.
      '/filter_form$/',
    // Forms ending with search_form.
      '/search_form$/',
    ];

    foreach ($excluded_patterns as $pattern) {
      if (preg_match($pattern, $form_id)) {
        return FALSE;
      }
    }

    // Get form mode and form lists.
    $form_mode = $config->get('form_mode') ?? 'all';
    $allowed_forms = $config->get('allowed_forms') ?? [];
    $excluded_forms = $config->get('excluded_forms') ?? [];

    // If excluded forms is set and this form is in it, don't add.
    if (!empty($excluded_forms) && in_array($form_id, $excluded_forms)) {
      return FALSE;
    }

    // Handle form mode.
    if ($form_mode === 'specific') {
      // Only allow forms in the allowed list.
      return in_array($form_id, $allowed_forms);
    }

    // Default mode 'all' - allow all forms except excluded.
    return TRUE;
  }

  /**
   * Adds the Smart Paste button to a form.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function addSmartPasteButton(array &$form, FormStateInterface $form_state): void {
    // Check if user has permission.
    if (!$this->currentUser->hasPermission('use smart paste')) {
      return;
    }

    // Find if there are any submit buttons in the form.
    $has_submit_button = $this->hasSubmitButton($form);

    // Only add Smart Paste button if a submit button exists.
    if (!$has_submit_button) {
      return;
    }

    // Ensure actions array exists.
    if (!isset($form['actions'])) {
      $form['actions'] = [
        '#type' => 'actions',
        '#weight' => 1000,
      ];
    }

    // Add the Smart Paste button to the actions.
    $form['actions']['smart_paste_button'] = [
      '#type' => 'button',
      '#value' => $this->t('🤖 Smart Paste'),
      '#attributes' => [
        'class' => ['smart-paste-button', 'button', 'button--secondary'],
        'data-form-id' => $this->getFormId($form, $form_state),
        'type' => 'button',
      ],
      '#attached' => [
        'library' => ['smart_paste/smart_paste'],
        'drupalSettings' => [
          'smart_paste' => [
            'extract_url' => $this->urlGenerator->generateFromRoute('smart_paste.extract_data'),
            'csrf_token' => $this->csrfToken->get('smart_paste_extract'),
          ],
        ],
      ],
      // Place after submit buttons.
      '#weight' => 10,
    ];

    // Add a container for status messages.
    $form['actions']['smart_paste_status'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['smart-paste-status'],
        'style' => 'display: none;',
      ],
      '#weight' => 9,
    ];
  }

  /**
   * Extracts structured data from unstructured text using AI.
   *
   * @param string $unstructured_data
   *   The unstructured data from the clipboard.
   * @param array $form_fields
   *   The form fields structure.
   *
   * @return array
   *   The extracted structured data.
   *
   * @throws \Exception
   *   If the AI processing fails.
   */
  public function extractData(string $unstructured_data, array $form_fields): array {
    $config = $this->configFactory->get('smart_paste.settings');
    $provider_id = $config->get('ai_provider');
    $model_id = $config->get('ai_model');

    if (empty($provider_id) || empty($model_id)) {
      throw new \Exception('AI provider or model not configured.');
    }

    try {
      // Get the AI provider.
      $provider = $this->aiProviderManager->createInstance($provider_id);

      if (!$provider->isUsable('chat')) {
        throw new \Exception('AI provider is not available for chat operations.');
      }

      // Generate the prompt.
      $prompt = $this->generatePrompt($form_fields, $unstructured_data);

      // Create chat input.
      $chat_input = new ChatInput([
        new ChatMessage('user', $prompt),
      ]);

      // Call the AI provider.
      $response = $provider->chat($chat_input, $model_id)->getNormalized();

      // Debug: Log the response type for troubleshooting.
      $this->logger->debug('AI response type: @type, content: @content', [
        '@type' => gettype($response),
        '@content' => is_string($response) ? $response : print_r($response, TRUE),
      ]);

      // Get the text content from the response.
      $response_text = '';

      if (is_array($response) && !empty($response)) {
        // Handle array of messages.
        foreach ($response as $message) {
          if (is_object($message) && method_exists($message, 'getText')) {
            $response_text .= $message->getText();
          }
          elseif (is_string($message)) {
            $response_text .= $message;
          }
        }
      }
      elseif (is_object($response) && method_exists($response, 'getText')) {
        // Handle single ChatMessage object.
        $response_text = $response->getText();
      }
      elseif (is_string($response)) {
        // Handle string response.
        $response_text = $response;
      }

      if (empty($response_text)) {
        throw new \Exception('Empty or unhandled response from AI provider. Response type: ' . gettype($response));
      }

      // Clean up the response text (remove any markdown formatting if present)
      $response_text = trim($response_text);
      if (strpos($response_text, '```json') !== FALSE) {
        // Extract JSON from markdown code block.
        preg_match('/```json\s*(.*?)\s*```/s', $response_text, $matches);
        if (!empty($matches[1])) {
          $response_text = trim($matches[1]);
        }
      }
      elseif (strpos($response_text, '```') !== FALSE) {
        // Extract from any code block.
        preg_match('/```\s*(.*?)\s*```/s', $response_text, $matches);
        if (!empty($matches[1])) {
          $response_text = trim($matches[1]);
        }
      }

      // Parse the JSON response.
      $extracted_data = json_decode($response_text, TRUE);

      if (json_last_error() !== JSON_ERROR_NONE) {
        $this->logger->error('Invalid JSON response from AI. Error: @error, Response: @response', [
          '@error' => json_last_error_msg(),
          '@response' => $response_text,
        ]);
        throw new \Exception('Invalid JSON response from AI: ' . json_last_error_msg() . '. Response: ' . substr($response_text, 0, 200));
      }

      if (!is_array($extracted_data)) {
        throw new \Exception('AI response is not a valid array');
      }

      return $extracted_data;

    }
    catch (\Exception $e) {
      $this->logger->error('Smart Paste extraction failed: @error', ['@error' => $e->getMessage()]);

      // For development/testing: provide a simple fallback.
      if (strpos($e->getMessage(), 'quota') !== FALSE || strpos($e->getMessage(), 'billing') !== FALSE) {
        // API quota exceeded - provide a simple demo extraction.
        return $this->createDemoExtraction($unstructured_data, $form_fields);
      }

      throw $e;
    }
  }

  /**
   * Creates a demo extraction for testing when AI is not available.
   */
  private function createDemoExtraction(string $unstructured_data, array $form_fields): array {
    $extracted_data = [];
    $lines = explode("\n", $unstructured_data);
    $all_text = strtolower($unstructured_data);

    foreach ($form_fields as $field_name => $field_info) {
      $field_type = $field_info['type'] ?? '';

      // Simple pattern matching for demo purposes.
      switch ($field_type) {
        case 'email':
          foreach ($lines as $line) {
            $line = trim($line);
            if (filter_var($line, FILTER_VALIDATE_EMAIL)) {
              $extracted_data[$field_name] = $line;
              break;
            }
          }
          break;

        case 'tel':
          foreach ($lines as $line) {
            $line = trim($line);
            if (preg_match('/[\+]?[\d\s\-\(\)]+/', $line) && strlen(preg_replace('/[^\d]/', '', $line)) >= 7) {
              $extracted_data[$field_name] = $line;
              break;
            }
          }
          break;

        case 'select':
          // Check for category keywords.
          if (strpos($field_name, 'category') !== FALSE) {
            if (strpos($all_text, 'business') !== FALSE || strpos($all_text, 'company') !== FALSE || strpos($all_text, 'corporation') !== FALSE) {
              $extracted_data[$field_name] = 'business';
            }
            elseif (strpos($all_text, 'education') !== FALSE || strpos($all_text, 'school') !== FALSE || strpos($all_text, 'university') !== FALSE) {
              $extracted_data[$field_name] = 'education';
            }
            elseif (strpos($all_text, 'technology') !== FALSE || strpos($all_text, 'software') !== FALSE || strpos($all_text, 'engineer') !== FALSE) {
              $extracted_data[$field_name] = 'technology';
            }
            elseif (strpos($all_text, 'healthcare') !== FALSE || strpos($all_text, 'doctor') !== FALSE || strpos($all_text, 'medical') !== FALSE) {
              $extracted_data[$field_name] = 'healthcare';
            }
            else {
              $extracted_data[$field_name] = 'personal';
            }
          }
          break;

        case 'checkbox':
          // Check for subscription/newsletter keywords.
          if (strpos($field_name, 'newsletter') !== FALSE || strpos($field_name, 'subscribe') !== FALSE) {
            $extracted_data[$field_name] = (strpos($all_text, 'subscribe') !== FALSE ||
                                           strpos($all_text, 'newsletter') !== FALSE ||
                                           strpos($all_text, 'updates') !== FALSE) ? 1 : 0;
          }
          break;

        case 'text':
        case 'textarea':
          // Handle specific field names.
          if (strpos($field_name, 'address') !== FALSE) {
            // Look for address-like patterns.
            foreach ($lines as $line) {
              $line = trim($line);
              if (preg_match('/\d+\s+[A-Za-z\s,]+/', $line) ||
                  strpos($line, 'St') !== FALSE ||
                  strpos($line, 'Ave') !== FALSE ||
                  strpos($line, 'Road') !== FALSE ||
                  strpos($line, 'Drive') !== FALSE) {
                $extracted_data[$field_name] = $line;
                break;
              }
            }
          }
          else {
            // Use first non-email, non-phone line for other text fields.
            foreach ($lines as $line) {
              $line = trim($line);
              if (empty($line)) {
                continue;
              }

              if (!filter_var($line, FILTER_VALIDATE_EMAIL) &&
                  !preg_match('/[\+]?[\d\s\-\(\)]+/', $line) &&
                  !isset($extracted_data[$field_name])) {
                $extracted_data[$field_name] = $line;
                break;
              }
            }
          }
          break;
      }
    }

    return $extracted_data;
  }

  /**
   * Generates a prompt for the AI model.
   *
   * @param array $form_fields
   *   The form fields structure.
   * @param string $unstructured_data
   *   The unstructured data.
   *
   * @return string
   *   The generated prompt.
   */
  protected function generatePrompt(array $form_fields, string $unstructured_data): string {
    $fields_description = '';

    foreach ($form_fields as $field_name => $field_info) {
      $type = $field_info['type'] ?? 'text';
      $label = $field_info['label'] ?? $field_name;
      $options = $field_info['options'] ?? [];

      $fields_description .= "- {$field_name} ({$label}): {$type}";

      if (!empty($options)) {
        $fields_description .= " - Options: " . implode(', ', $options);
      }

      $fields_description .= "\n";
    }

    $prompt = "You are an AI assistant that extracts structured data from unstructured text to fill form fields.\n\n";
    $prompt .= "FORM FIELDS:\n{$fields_description}\n";
    $prompt .= "UNSTRUCTURED DATA:\n{$unstructured_data}\n\n";
    $prompt .= "INSTRUCTIONS:\n";
    $prompt .= "1. Extract relevant information from the unstructured data\n";
    $prompt .= "2. Map the extracted information to the appropriate form fields\n";
    $prompt .= "3. Return ONLY a valid JSON object with field names as keys and extracted values as values\n";
    $prompt .= "4. If no relevant data is found for a field, omit it from the JSON\n";
    $prompt .= "5. For select/radio fields, use only the provided options\n";
    $prompt .= "6. Do not include any explanations or additional text\n\n";
    $prompt .= "JSON RESPONSE:";

    return $prompt;
  }

  /**
   * Finds the Save button (submit button) in a form.
   *
   * @param array $form
   *   The form array.
   * @param array $path
   *   Current path in the form (for recursion).
   *
   * @return array|null
   *   Array with 'path' and 'key' of the Save button, or NULL if not found.
   */
  protected function findSaveButton(array $form, array $path = []): ?array {
    foreach ($form as $key => $element) {
      // Skip non-array elements and elements starting with #.
      if (!is_array($element) || strpos($key, '#') === 0) {
        continue;
      }

      // Check if this element is a submit button.
      if (isset($element['#type']) && $element['#type'] === 'submit') {
        // Common Save button values.
        $save_button_values = [
          'Save',
          'Submit',
          'Create',
          'Update',
          'Save and continue',
          'Save configuration',
          'Save settings',
        ];

        $button_value = $element['#value'] ?? '';
        if (is_object($button_value)) {
          $button_value = $button_value->render();
        }

        // Check if this looks like a Save button.
        foreach ($save_button_values as $save_value) {
          if (stripos($button_value, $save_value) !== FALSE) {
            return [
              'path' => $path,
              'key' => $key,
            ];
          }
        }
      }

      // Recursively search in nested elements.
      $nested_result = $this->findSaveButton($element, array_merge($path, [$key]));
      if ($nested_result) {
        return $nested_result;
      }
    }

    return NULL;
  }

  /**
   * Gets a reference to the form container that contains the Save button.
   *
   * @param array $form
   *   The form array.
   * @param array $save_button_location
   *   The location of the Save button (path and key).
   *
   * @return array|null
   *   Reference to the container, or NULL if not found.
   */
  protected function &getFormContainer(array &$form, array $save_button_location): ?array {
    $container = &$form;

    // Navigate to the container using the path.
    foreach ($save_button_location['path'] as $path_element) {
      if (isset($container[$path_element]) && is_array($container[$path_element])) {
        $container = &$container[$path_element];
      }
      else {
        $null_reference = NULL;
        return $null_reference;
      }
    }

    return $container;
  }

  /**
   * Checks if a form has any submit buttons.
   *
   * @param array $form
   *   The form array.
   *
   * @return bool
   *   TRUE if the form has submit buttons, FALSE otherwise.
   */
  protected function hasSubmitButton(array $form): bool {
    foreach ($form as $key => $element) {
      // Skip non-array elements and elements starting with #.
      if (!is_array($element) || strpos($key, '#') === 0) {
        continue;
      }

      // Check if this element is a submit button.
      if (isset($element['#type']) && $element['#type'] === 'submit') {
        return TRUE;
      }

      // Recursively search in nested elements.
      if ($this->hasSubmitButton($element)) {
        return TRUE;
      }
    }

    return FALSE;
  }

  /**
   * Gets a reliable form ID for the Smart Paste button data attribute.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return string
   *   The form ID to use.
   */
  private function getFormId(array $form, FormStateInterface $form_state): string {
    // Try to get the form ID from various sources.
    // First, try the form's #id attribute.
    if (!empty($form['#id'])) {
      return $form['#id'];
    }

    // Try to get from form state build info.
    $build_info = $form_state->getBuildInfo();
    if (!empty($build_info['form_id'])) {
      return $build_info['form_id'];
    }

    // Try to get from form object.
    if (!empty($build_info['callback_object'])) {
      $form_object = $build_info['callback_object'];
      if (method_exists($form_object, 'getFormId')) {
        return $form_object->getFormId();
      }
    }

    // Fallback to a generic form selector.
    return 'drupal-form';
  }

}
