<?php

namespace Drupal\Tests\smart_paste\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * Tests the Smart Paste module functionality.
 *
 * @group smart_paste
 */
class SmartPasteTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = ['smart_paste', 'ai', 'key'];

  /**
   * A user with administration privileges.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create admin user.
    $this->adminUser = $this->drupalCreateUser([
      'administer smart paste',
      'use smart paste',
    ]);
  }

  /**
   * Tests the Smart Paste admin form.
   */
  public function testAdminForm() {
    // Login as admin user.
    $this->drupalLogin($this->adminUser);

    // Access the admin form.
    $this->drupalGet('admin/config/services/smart-paste');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Smart Paste Settings');

    // Test form elements exist.
    $this->assertSession()->fieldExists('enabled');
    $this->assertSession()->fieldExists('ai_provider');
    $this->assertSession()->fieldExists('form_mode');
  }

  /**
   * Tests that Smart Paste buttons appear on forms when enabled.
   */
  public function testSmartPasteButtonAppearance() {
    // Enable Smart Paste and configure it properly.
    $config = $this->config('smart_paste.settings');
    $config->set('enabled', TRUE);
    // Allow on all forms instead of specific forms only.
    $config->set('form_mode', 'all');
    // Set a dummy provider to satisfy configuration requirements.
    $config->set('ai_provider', 'test_provider');
    // Set a dummy model.
    $config->set('ai_model', 'test_model');
    $config->save();

    // Create a user with Smart Paste permission.
    $user = $this->drupalCreateUser(['use smart paste']);
    $this->drupalLogin($user);

    // Visit a form that should have Smart Paste (user edit form).
    $this->drupalGet('user/' . $user->id() . '/edit');

    // Check if Smart Paste button is present.
    $this->assertSession()->elementExists('css', '.smart-paste-button');
  }

  /**
   * Tests that Smart Paste buttons do not appear when disabled.
   */
  public function testSmartPasteButtonNotAppearingWhenDisabled() {
    // Ensure Smart Paste is disabled but configure other settings.
    $config = $this->config('smart_paste.settings');
    $config->set('enabled', FALSE);
    // Set consistent configuration.
    $config->set('form_mode', 'all');
    // Set a dummy provider.
    $config->set('ai_provider', 'test_provider');
    // Set a dummy model.
    $config->set('ai_model', 'test_model');
    $config->save();

    // Create a user with Smart Paste permission.
    $user = $this->drupalCreateUser(['use smart paste']);
    $this->drupalLogin($user);

    // Visit a form.
    $this->drupalGet('user/' . $user->id() . '/edit');

    // Check that Smart Paste button is not present.
    $this->assertSession()->elementNotExists('css', '.smart-paste-button');
  }

  /**
   * Tests permissions.
   */
  public function testPermissions() {
    // Create a user without Smart Paste permission.
    $user = $this->drupalCreateUser();
    $this->drupalLogin($user);

    // Should not be able to access admin form.
    $this->drupalGet('admin/config/services/smart-paste');
    $this->assertSession()->statusCodeEquals(403);
  }

}
